
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/** @author  John Miller
 *  @version 1.6
 *  @date    Wed Oct  9 18:21:13 EDT 2019
 *  @see     LICENSE (MIT style license file).
 */

package scalation.analytics

import scala.collection.mutable.Set

import scalation.linalgebra.MatriD

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/** The `NoFeatureSelectionMat` trait is for modeling techniques that do not support
 *  feature selection (adding/remove variables from the model).  For example,
 *  `SimpleRegression` only has one feature/predictor variable, so feature
 *  selection makes no sense for this modeling technique.  Therefore the 'buildModel'
 *  method specified in `PredictorMat` and is called by 'forwardSel' and 'backwardElim'
 *  is set to throw an exception.
 */
trait NoFeatureSelectionMat
{
    def buildModel (x_cols: MatriD): PredictorMat =
    {
        throw new UnsupportedOperationException ("buildModel: this model does not support feature selection")
    } // buildModel

} // NoFeatureSelectionMat trait

